#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2012 Liricus SRL <info@liricus.com.ar>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Fra


#===============================================================================
# DOCS
#===============================================================================

"""It's a server for image display from several social networks.

Lets you create custom "bubbles" to show certain pictures, and share them.

"""

#===============================================================================
# IMPORTS
#===============================================================================

import inspect
import sys
import tempfile
import os
import argparse
import datetime
import string
import hashlib
import shutil
import pprint

from socialframe_server import settings


#===============================================================================
# METADATA
#===============================================================================

__prj__ = "socialframe_server"

__version__ = ("0", "4", "b")

__str_version__ = ".".join(__version__)

__short_description__ = __doc__.splitlines()[0]

__description__ = __doc__

__author__ = "Liricus SRL"

__email__ = "info@liricus.com.ar"

__url__ = "http://bitbucket.org/liricus/socialframe"

__download_url__ = "{}/downloads/{}-{}.tar.gz".format(__url__, __name__,
                                                      __str_version__)

__license__ = "GPL3"

__keywords__ = "socialframe socialone social network photo"

__classifiers__ = (
    "Topic :: Utilities",
    "License :: OSI Approved :: GNU General Public License (GPL)",
    "Programming Language :: Python :: 2",
)

__dependencies__ = ("flask",
                    "flask-peewee",
                    "Flask-Mail",
                    "Flask-Uploads",
                    "Flask-Cache",
                    "pillow")


#===============================================================================
# STAR FUNCTIONS
#===============================================================================


def _construct_database(db):
    if db["schema"] in "sqlite3":
        return {"engine": "peewee.SqliteDatabase", "name": db["name"]}
    elif db["schema"] == "memory":
        return {"engine": "peewee.SqliteDatabase", "name": ":memory:"}
    elif db["schema"] == "mysql":
        return {
            "engine": "peewee.MySQLDatabase", "name": db["name"],
            "user": db["user"], "passwd": db["password"],
            "host": db["host"], "port": int(db["port"]),
            'threadlocals': False
        }
    elif db["schema"] == "postgres":
        return {
            "engine": "peewee.PostgresqlDatabase", "name": db["name"],
            "user": db["user"], "password": db["password"],
            "host": db["host"], "port": int(db["port"]),
        }
    msg = "Invalid schema '{}'".format(db["schema"])
    raise ValueError(msg)
    
    
def init_settings(user_settings):
    
    for k, v in vars(user_settings).items():
        setattr(settings, k, v)
        if(k == "DB"):
            setattr(settings, "DATABASE", _construct_database(v))
        elif(k == "SECRET_KEY"):
            setattr(settings, "secret_key", v)
    
    from socialframe_server import api        
    from socialframe_server import models
    from socialframe_server import jobs
    for model in models.autocreatetables():
        model.create_table(fail_silently=True)
        

def run_server(env, *args):
    from socialframe_server import core
    core.APP.run(host=settings.HOST,
                 port=settings.PORT,
                 debug=settings.DEBUG)


def run_jobs(env, *args):
    from socialframe_server import core
    core.run_jobs()
 

def run_make_env(env, *args):
    
    def mkdir(d):
        if not os.path.exists(d):
            os.makedirs(d)
    
    from socialframe_server import data
    
    wd = env
    mkdir(wd)
    
    bkname = "_bk_" + datetime.datetime.now().strftime("%Y%m%d_%H%m%s.%f")
    bkdir = os.path.join(wd, bkname)
    
    # settings.py
    sfp = os.path.join(wd, "settings.py")
    if os.path.exists(sfp):
        mkdir(bkdir)
        shutil.move(sfp, bkdir)
    secret_key = hashlib.sha256().hexdigest()
    with open(data.get("settings.template.py")) as fp:
        src = string.Template(fp.read()).safe_substitute(SECRET_KEY=secret_key)
    with open(sfp, "w") as fp:
        fp.write(src)
    print("New settings created on '{}'. Please configure it".format(sfp))

    # this creates a new wsgi file
    sfp = os.path.join(wd, "socialframe.wsgi")
    if os.path.exists(sfp):
        mkdir(bkdir)
        shutil.move(sfp, bkdir)
    src = data.get("socialframe.template.wsgi")
    shutil.copyfile(src, sfp)
    print("New wsgi file created on '{}'. Please configure it".format(sfp))

    # this creates a new test file and his test
    sfp = os.path.join(wd, "test.py")
    if os.path.exists(sfp):
        mkdir(bkdir)
        shutil.move(sfp, bkdir)
    src = data.get("test.template.py")
    shutil.copyfile(src, sfp)
    print("New test file created on '{}'.".format(sfp))
    
    # this creates a new test data dir and his test
    sfp = os.path.join(wd, "test_data")
    if os.path.exists(sfp):
        mkdir(bkdir)
        shutil.move(sfp, bkdir)
    src = data.get("test_data")
    shutil.copytree(src, sfp)
    print("New test data dumped in: '{}'.".format(sfp))
    
    
    # if backup is created anounce it
    if os.path.exists(bkdir):
        print("ALL OLD FILES IS MOVED TO: '{}'".format(bkdir))


def run_show_vars(env, *args):
    from socialframe_server import settings
    msg = []
    for k in args:
        if hasattr(settings, k):
            v = pprint.pformat(getattr(settings, k))
            msg.append("{} = {}".format(k, v))
    print "\n".join(msg)
    print ""
    

#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print(__doc__)
    
